const items = [
  { id: 0, title: 'Root item' },
  { id: 1, superIds: [ 0 ], priorIds: { 0: 2 }, title: 'Second subitem' },
  { id: 2, superIds: [ 9, 3, 0 ], title: 'First subitem' },
  { id: 3, superIds: [ 1, 2, 9 ], title: 'Shared subitem of 1 & 2 & 3' },
  { id: 4, superIds: [ 1 ], title: 'First subitem of first subitem' },
  { id: 5, superIds: [ 1 ], priorIds: { 1: 6 }, title: 'Third subitem of first subitem' },
  { id: 6, superIds: [ 1 ], priorIds: { 1: 4 }, title: 'Second subitem of first subitem' },
  { id: 7, superIds: [ 2 ], title: 'First subitem of the second subitem' },
  { id: 8, superIds: [ 3 ], title: 'Subitem of the shared subitem' },
  { id: 9, superIds: [ 0 ], title: 'Third subitem' },
  { id: 10, superIds: [ 9 ], title: 'First subitem of third subitem' },
  { id: 11, superIds: [ 0, 11 ], title: 'Item in itself' }
];

const ownerIds = {};

function render() {
  document.body.innerHTML = '';

  const dataDiv = document.createElement('div');
  dataDiv.classList.add('data');
  document.body.appendChild(dataDiv);

  document.body.appendChild(document.createElement('hr'));

  for (const item of recurse(items, ownerIds)) {
    const metaDiv = document.createElement('div');
    metaDiv.classList.add('meta');
    metaDiv.style.paddingLeft = `${item.level}em`;
    metaDiv.textContent = `id: ${item.item.id} | superId: ${item.superId} | superIds: ${item.item.superIds} | priorIds: ${JSON.stringify(item.item.priorIds)} | ${item.state}`;

    document.body.appendChild(metaDiv);

    const itemDiv = document.createElement('div');
    itemDiv.classList.add('item');
    itemDiv.style.paddingLeft = `${item.level}em`;

    switch (item.state) {
      case 'expandable-implicitly-expanded':
      case 'expandable-explicitly-expanded': {
        const collapseButton = document.createElement('button');
        collapseButton.classList.add('directory-opened');
        collapseButton.textContent = '-';

        collapseButton.addEventListener('click', _ => {
          items.find(i => i.id === item.item.id).state = 'collapsed';
          render();
        });

        itemDiv.appendChild(collapseButton);
        break;
      }

      case 'expandable-implicitly-collapsed':
      case 'expandable-explicitly-collapsed': {
        const expandButton = document.createElement('button');
        expandButton.classList.add('directory-closed');
        expandButton.textContent = '+';

        expandButton.addEventListener('click', _ => {
          ownerIds[item.item.id] = item.superId;
          items.find(i => i.id === item.item.id).state = 'expanded';
          render();
        });

        itemDiv.appendChild(expandButton);
        break;
      }

      case 'unexpandable': {
        const openButton = document.createElement('button');
        openButton.classList.add('file');
        openButton.textContent = '>';

        openButton.addEventListener('click', _ => {
          alert(item.item.title);
        });

        itemDiv.appendChild(openButton);
        break;
      }

      default: {
        throw new Error(`Invalid item state '${item.state}'.`);
      }
    }

    const titleSpan = document.createElement('span');
    titleSpan.classList.add('title');
    titleSpan.textContent = item.item.title;

    itemDiv.appendChild(titleSpan);

    document.body.appendChild(itemDiv);
  }

  /* Update after the loop to capture changes made to input arguments by `recurse` */

  const ownersDetails = document.createElement('details');
  ownersDetails.open = true;

  const ownersSummary = document.createElement('summary');
  ownersSummary.textContent = 'Owners';

  ownersDetails.appendChild(ownersSummary);

  const ownersCode = document.createElement('code');
  ownersCode.textContent = JSON.stringify(ownerIds);

  ownersDetails.appendChild(ownersCode);

  dataDiv.appendChild(ownersDetails);

  const itemsDetails = document.createElement('details');
  itemsDetails.open = true;

  const itemsSummary = document.createElement('summary');
  itemsSummary.textContent = 'Items';

  itemsDetails.appendChild(itemsSummary);

  for (const item of items) {
    const itemCode = document.createElement('code');
    itemCode.textContent = JSON.stringify(item);
    itemsDetails.appendChild(itemCode);
  }

  dataDiv.appendChild(itemsDetails);
}

window.addEventListener('load', _ => {
  render();
});
